#!/bin/sh
# $Header: /magellan-cvs/magellan-src/mkinitrd-magellan/livecd/linuxrc.sh,v 1.3 2008/01/14 19:32:52 niro Exp $

# loads given kernel modules
load_kernel_modules()
{
	local type="$1"
	local module
	local args
	
	echo -e "-- Loading ${type} modules:"
	( cat /etc/modules.${type}; echo) | while read module args
	do
		case "${module}" in
			\#*|"") continue ;;
		esac

		if [ -e /lib/${module}.ko ]
		then
			insmod -f /lib/${module}.ko ${args} > /dev/null 2>&1
			[ $? -eq 0 ] && echo -e "--      ${module}"
		fi
	done
	echo
}

# check_drives $devices $fstype
check_drives()
{
	local devices="$1"
	local fs="$2"

	for dev in ${devices}
	do
		mount -t ${fs} ${dev} /sysroot/mnt/cdrom >/dev/null 2>&1
		media_found=$?

		# if mount was ok check if cdid exists
		if [ "${media_found}" -eq 0 ]
		then
			[ -e /sysroot/mnt/cdrom/livecdrootfs.sqsh ] && media_id_ok=$?
			umount /sysroot/mnt/cdrom
	
			if [ "${media_id_ok}" -eq 0 ]
			then
				echo -e "Magellan boot device found."
				BOOT_DEVICE="${dev}"
				FSTYPE="${fs}"
				break
			fi
		fi
	done
}

## some defaults ##
DOSCSI=no
DOUSB=no
DOSATA=no
DOFB=no
NOPATA=no
FORCED_ROOTFS=""

## starts here ##

# mount essential filesystems
mount -t proc proc /proc
mount -t sysfs sysfs /sys

# silencio
echo 0 > /proc/sys/kernel/printk

# check command line for params
for i in $(cat /proc/cmdline)
do
	case "${i}" in
		doscsi)
			DOSCSI=yes
			;;
		dousb)
			DOUSB=yes
			;;
		dosata)
			DOSATA=yes
			;;
		dofb)
			DOFB=yes
			;;
		nopata)
			NOPATA=yes
			;;
		rootfs=*)
			shift
			FORCED_ROOTFS="${1#*=}"
			;;
	esac
done

# now load all needed modules
load_kernel_modules "generic"
[ "${NOPATA}" != "yes" ] && load_kernel_modules "pata"
[ "${DOSCSI}" = "yes" ] && load_kernel_modules "scsi"
[ "${DOUSB}" = "yes" ] && load_kernel_modules "usb"
[ "${DOSATA}" = "yes" ] && load_kernel_modules "sata"
[ "${DOFB}" = "yes" ] && load_kernel_modules "framebuffer"

# create newroot mount point
mkdir -p /sysroot
# fake a new filesystem (also needed with an initramfs!!)
mount -t tmpfs tmpfs /sysroot

mkdir -p /sysroot/mnt/cdrom
mkdir -p /sysroot/mnt/cloop
mkdir -p /sysroot/mnt/magellan


# needed for the usbstick, the timeout maybe to short
if [ "${DOUSB}" = "yes" ]; then
	echo "-- Waiting 8 seconds for possible usb mount ..."
	sleep 8
fi

# searching cdrom with cdid
BOOT_DEVICE=""
FSTYPE=""

echo "Searching for a boot device ..."

# searching usbstick
if [ "${DOUSB}" = "yes" ]; then
	ROOTFS="vfat"
	[[ -n ${FORCED_ROOTFS} ]] && ROOTFS="${FORCED_ROOTFS}"
	check_drives "/dev/sd*" "${ROOTFS}"
	check_drives "/dev/sg*" "${ROOTFS}"
fi

# for cdrom_device in $CDROM_LIST (ide)
ROOTFS="iso9660"
[[ -n ${FORCED_ROOTFS} ]] && ROOTFS="${FORCED_ROOTFS}"
check_drives "/dev/cdroms/*" "${ROOTFS}"
check_drives "/dev/hd*" "${ROOTFS}"

# scsi cdroms
if [ "${DOSCSI}" = "yes" ] || [ "${DOSATA}" = "yes" ]; then
	ROOTFS="iso9660"
	[[ -n ${FORCED_ROOTFS} ]] && ROOTFS="${FORCED_ROOTFS}"
	check_drives "/dev/sr*" "${ROOTFS}"
	check_drives "/dev/sg*" "${ROOTFS}"
	check_drives "/dev/scd*" "${ROOTFS}"
fi

# mount our rootfs from cdrom
if [ "${BOOT_DEVICE}" = "" ]
then
	echo "No Magellan boot CD/USBStick found!!!"
	ash
	exit 1
else
	echo -e "Booting from: $BOOT_DEVICE ..."
	mount -o ro -t "${FSTYPE}" "${BOOT_DEVICE}" /sysroot/mnt/cdrom >/dev/null 2>&1

	# write our boot_device to /mnt/.bootdev for later use (mtab)
	echo "BOOTDEV=${BOOT_DEVICE}" > /sysroot/.bootdev
	echo "FSTYPE=${FSTYPE}" >> /sysroot/.bootdev

	# mount squashfs
	echo -e "Mounting squashfs system image ..."
	echo
	[ ! -e /dev/loop0 ] && mknod /dev/loop0 b 7 0
	mount -o loop,ro -t squashfs /sysroot/mnt/cdrom/livecdrootfs.sqsh /sysroot/mnt/cloop

	# symlinking cloop rootfs to /mnt as newroot
	cd /sysroot

	for x in bin sbin lib lib64 boot usr opt
	do
		[ -d mnt/cloop/${x} ] && ln -s mnt/cloop/${x} ${x}
	done
	mkdir -p initrd proc tmp sys var/tmp dev media

	# make /sysroot/dev/console & /mnt/dev/null
	[ ! -e /sysroot/dev/console ] && mknod -m 600 /sysroot/dev/console c 5 1
	[ ! -e /sysroot/dev/null ] && mknod -m 666 /sysroot/dev/null c 1 3

	(cd /sysroot/mnt/cloop; cp -a etc root home var /sysroot)

	# ensure the right permissions
	chmod 1777 /sysroot/tmp
	chmod 1777 /sysroot/var/tmp
fi

cd /

# de-silencio
echo 3 > /proc/sys/kernel/printk

echo "-- Switching to real sysroot ..."
umount /sys
umount /proc

exec run-init /sysroot /sbin/init $@ </sysroot/dev/console >/sysroot/dev/console
